@@ -1,4 +1,4 @@
-/* $Id: OSnLNode.h 2738 2009-06-20 06:39:18Z kmartin $ */
+/* $Id: OSnLNode.h 3815 2010-11-03 23:46:16Z Gassmann $ */
 /** @file OSnLNode.h
  * \brief This file defines the OSnLNode class along with its derived classes.
  *
@@ -17,57 +17,30 @@
 
 #ifndef OSNLNODE_H
 #define OSNLNODE_H
+
 #include "OSConfig.h"
 #include "OSErrorClass.h"
 #include<iostream>
 #include<vector>
 #include <map> 
 
+
 #ifdef COIN_HAS_CPPAD  
 #include<cppad/cppad.hpp>
 using CppAD::AD;
-using CppAD::vector;
+//using CppAD::vector;
 typedef AD<double>  ADdouble;
 #else
 typedef double  ADdouble;
-using std::vector;
+//using std::vector;
 #endif
 
-/** \enum OP_CODES we give a name  or op code to each of the values of
- * inodeInt (number identifying the type of node) -- this is
- * easier to work with
- */
- enum OP_CODES{
-	OS_PLUS = 1001,
-	OS_SUM = 1002,
-	OS_MINUS = 1003,
-	OS_NEGATE = 1004,
-	OS_TIMES = 1005,
-	OS_DIVIDE = 1006,
-	OS_POWER = 1009,
-	OS_PRODUCT = 1010,
-	OS_ABS = 2001,
-	OS_SQUARE = 2005,
-	OS_SQRT = 2006,
-	OS_LN = 2007,
-	OS_EXP = 2010,
-	OS_SIN = 3001,
-	OS_COS = 3002,
-	OS_MIN = 4010,
-	OS_MAX = 4011,
-	OS_ERF = 4625,
-	OS_NUMBER = 5001,
-	OS_PI = 5003,
-	OS_E = 5004,
-	OS_VARIABLE = 6001,
-	OS_IF = 7001,
-	OS_ALLDIF = 7016
-};
 
+ 
 /*! \class OSnLNode OSnLNode.h "OSnLNode.h"
  *  \brief The OSnLNode Class.
  *
- * @author  Robert Fourer, Jun Ma, Kipp Martin, 
+ * @author  Horand Gassmann, Jun Ma, Kipp Martin, 
  * @version 1.0, 10/05/2005
  * @since   OS1.0
  * 
@@ -82,22 +55,22 @@ class OSnLNode{
 public:	
 
 	
-	/** nodeName holds the OSnLNode name */
-	std::string snodeName;
 	
-	/**  numberOfChildren is the number of OSnLNode child elements*/
+	/**  numberOfChildren is the number of OSnLNode child elements
+	 * set to -1 if the number is not fixed, e.g., for a sum node
+	 */
 	unsigned int inumberOfChildren;	
 
 	/**  ssymbolInt is the unique integer assigned to the OSnLNode*/	
 	int inodeInt;
 
 	/** inodeType is the number of children the node has, it is set
-	 * to -1 if the number is not known a prior, e.g. a sum node
+	 * to -1 if the number is not known a priori, e.g., a sum node
 	 */	
 	int inodeType;
 	
 	/**
-	 * m_mChildren holds all the operands, that is nodes that the current node operate on.
+	 * m_mChildren holds all the operands, that is, nodes that the current node operate on.
 	 */
 	OSnLNode **m_mChildren;
 	
@@ -120,9 +93,9 @@ public:
 	
 	/**
 	 *
-	 * @return the value of snodeName
+	 * @return the value of operator name
 	 */	
-	virtual std::string getTokenName();
+	virtual std::string getTokenName() = 0;
 	
 	/**
 	 * <p>
@@ -236,8 +209,11 @@ public:
 	 *
 	 * @return the expression tree.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD) = 0;
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) = 0 ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) = 0;
+#endif
 	/**
 	 * <p>
 	 * Create or clone a node of this type.
@@ -250,8 +226,6 @@ public:
 	 */
 	virtual OSnLNode *cloneOSnLNode() = 0;
 	
-
-	
 	/**
 	 * default constructor.
 	 */
@@ -261,7 +235,11 @@ public:
 	 * default destructor.
 	 */	
 	virtual ~OSnLNode();
-	
+
+	/**
+	 * A function to check for the equality of two objects
+	 */
+	bool IsEqual(OSnLNode *that);
 };//end OSnLNode
 
 /*! \class OSnLNodePlus
@@ -275,7 +253,6 @@ public:
  * The in-memory representation of the OSnL element <plus>
  * 
  */
-
 class OSnLNodePlus : public OSnLNode{  
 public:
 	/**
@@ -288,6 +265,12 @@ public:
 	 */	
 	~OSnLNodePlus();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodePlus::calculateFunction(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -298,7 +281,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif
 	
 	/*! \fn OSnLNode *cloneOSnLNode(double *x) 
 	 *  \brief The implementation of the virtual functions. 
@@ -332,6 +319,12 @@ public:
 	 */	
 	~OSnLNodeSum();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeSum::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -348,11 +341,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-	
-
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif	
 };//end OSnLNodeSum
 
 /*! \class OSnLNodeMax
@@ -378,6 +371,12 @@ public:
 	 */	
 	~OSnLNodeMax();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeMax::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -393,11 +392,12 @@ public:
 	/*! \fn double OSnLNodeMax::constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
-	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
-
+	 */		
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 };//end OSnLNodeMax
 
 /*! \class OSnLNodeMin
@@ -423,6 +423,12 @@ public:
 	 */	
 	~OSnLNodeMin();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeMin::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -439,9 +445,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif
 
 };//end OSnLNodeMin
 
@@ -469,6 +477,13 @@ public:
 	 */	
 	~OSnLNodeMinus();
 	
+	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeMinus::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -485,9 +500,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif
 
 };//end OSnLNodeMinus
 
@@ -514,6 +531,12 @@ public:
 	 */	
 	~OSnLNodeNegate();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeNegate::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -531,9 +554,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif
 };//end OSnLNodeNegate
 
 
@@ -560,6 +585,12 @@ public:
 	 */	
 	~OSnLNodeTimes();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeTimes::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -576,9 +607,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD) ;
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD) ;
+#endif
 
 };//end OSnLNodeTimes
 
@@ -606,6 +639,13 @@ public:
 	 */	
 	~OSnLNodeDivide();
 	
+	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeDivide::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -622,8 +662,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 };//end OSnLNodeDivide
 
 /*! \class OSnLNodePower
@@ -649,6 +692,12 @@ public:
 	 */	
 	~OSnLNodePower();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodePower::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -665,9 +714,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 };//end OSnLNodePower
 
@@ -694,6 +745,12 @@ public:
 	 */	
 	~OSnLNodeProduct();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeProduct::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -710,9 +767,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 };//end OSnLNodeProduct
 
@@ -740,6 +799,12 @@ public:
 	 */	
 	~OSnLNodeLn();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeLn::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -756,9 +821,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 	
 
 };//end OSnLNodeLn
@@ -786,6 +853,12 @@ public:
 	 */	
 	~OSnLNodeSqrt();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeSqrt::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -801,9 +874,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 	
 
 };//end OSnLNodeSqrt
@@ -831,6 +906,12 @@ public:
 	 */	
 	~OSnLNodeSquare();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeSquare::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -847,8 +928,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 };//end OSnLNodeSquare
 
 /*! \class OSnLNodeCos
@@ -874,6 +958,12 @@ public:
 	 */	
 	~OSnLNodeCos();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeCos::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -890,8 +980,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 };//end OSnLNodeCos
 
 /*! \class OSnLNodeSin
@@ -917,6 +1010,12 @@ public:
 	 */	
 	~OSnLNodeSin();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeSin::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -933,8 +1032,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 };//end OSnLNodeSin
 
 /*! \class OSnLNodeExp
@@ -960,6 +1062,12 @@ public:
 	 */	
 	~OSnLNodeExp();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeExp::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -970,8 +1078,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif	
 	/*! \fn OSnLNode *cloneOSnLNode(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a point to a new OSnLNode of the proper type.
@@ -1004,6 +1115,12 @@ public:
 	 */	
 	~OSnLNodeAbs();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeAbs::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -1020,9 +1137,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 };//end OSnLNodeAbs
 
@@ -1050,6 +1169,12 @@ public:
 	 */	
 	~OSnLNodeErf();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeErf::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -1066,8 +1191,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif	
 };//end OSnLNodeErf
 
 
@@ -1096,6 +1224,12 @@ public:
 	 */	
 	~OSnLNodeIf();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeIf::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -1112,9 +1246,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);	
-	
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 };//end OSnLNodeIf
 
@@ -1153,16 +1289,15 @@ public:
 	
 	/**
 	 *
-	 * @return a string token that corresponds to the OSnLNode.
-	 */
-	virtual std::string getTokenNumber();
-	
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
 	
 	/**
 	 *
 	 * @return a string token that corresponds to the OSnLNode.
 	 */
-	virtual std::string getTokenName();
+	virtual std::string getTokenNumber();
 	
 	/**
 	 *
@@ -1187,8 +1322,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif	
 
 
 };//end OSnLNodeNumber
@@ -1218,6 +1356,7 @@ public:
 	 */	
 	~OSnLNodeE();
 	
+
 	/**
 	 *
 	 * @return a string token that corresponds to the OSnLNode.
@@ -1254,8 +1393,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 
 };//end OSnLNodeE
@@ -1284,6 +1426,7 @@ public:
 	 */	
 	~OSnLNodePI();
 	
+
 	/**
 	 *
 	 * @return a string token that corresponds to the OSnLNode.
@@ -1320,8 +1463,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif	
 
 
 };//end OSnLNodePI
@@ -1359,6 +1505,8 @@ public:
 	 */	
 	~OSnLNodeVariable();
 	
+
+	
 	virtual void getVariableIndexMap(std::map<int, int> *varIdx);
 	
 	/**
@@ -1396,8 +1544,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);	
-	
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 
 };//end OSnLNodeVariable
@@ -1426,6 +1577,12 @@ public:
 	 */	
 	~OSnLNodeAllDiff();
 	
+	/**
+	 *
+	 * @return the value of operator name
+	 */	
+	virtual std::string getTokenName();
+	
 	/*! \fn double OSnLNodeAllDiff::double(double *x) 
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a double.
@@ -1442,10 +1599,11 @@ public:
 	 *  \brief The implementation of the virtual functions. 
 	 *  \return a ADdouble.
 	 */	
-
-	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, vector< ADdouble > *XAD);
-
-
+#ifdef COIN_HAS_CPPAD
+	virtual ADdouble constructADTape(std::map<int, int> *ADIdx, CppAD::vector< ADdouble > *XAD);
+#else
+	virtual double constructADTape(std::map<int, int> *ADIdx, std::vector< double > *XAD);
+#endif
 
 };//end OSnLNodeAllDiff
 
